"""SAM→RGBA image tools.

Import path: ``core``
Project distribution: ``sam2rgba``
"""
__all__ = [
    "build_rgba_methylation",
    "build_rgba_nomethylation",
    "cli",
    "image_encoder",
    "methylation_loader",
    "sam_parser",
    "shared",
    "utils",
]
__version__ = "0.1.0"
